/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
#ifndef _MOVIE_H
#define _MOVIE_H

#include <movie.h>
#include "audio.h"

class Movie
{
 public:
  Movie();
  virtual ~Movie();

  int getHeight() {return _height;}
  int getWidth() {return _width;} 
  double getRate() {return _rate;} 
  int getFrameSize() {return _frameSize;}
  long getImageID(){ return _imageID; }
  void setImageID(MVid);
  long getMovieID(){ return _movieID; }
  void setMovieID(MVid);
  long getAudioID(){ return _audioID; }
  void setAudioID(MVid);
  MVframe getNumAudio() { return _numAudio;} 
  void setNumAudio(MVframe a) { _numAudio = a;} 
  MVframe getNumFrames() {return _numFrames;} 
  void setNumFrames(MVframe n) { _numFrames = n;} 
  void *getFrame(int);
  int setFrame(MVframe, void *);
  int setMovie(const char *name);
  char *getName();
  char *getCompressionType();
  void addAudio(Audio *);
  void startFrame() { _currentFrame = 0;} 
  
 protected:
  MVid _movieID;
  MVid _imageID;
  MVid _audioID;
  
  int _width;
  int _height;
  int _currentFrame;
  int _optimizeMovie;
  double _rate;
  MVframe _numFrames;
  int _frameSize;
  char *_movieName;
  MVframe _numAudio;
  char *_compressType;
};

class NewMovie : public Movie
{
 public:
  
  NewMovie(const char *name, double rate = 4.0, 
	   int w = 320, int h = 240, 
	   const char *c = "MVC1"  );
  NewMovie(Movie *, const char *);
  ~NewMovie();
  void nextFrame(void *buf) {setFrame(_currentFrame++, buf);} 
};

class OldMovie : public Movie
{
 public:
  
  OldMovie(const char *name);
  ~OldMovie();
  void nextFrame(void *buf) {buf = getFrame(_currentFrame++);} 
};  

#endif
