/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/*
    vlan.h - Routines to help talk to VLAN network.
    
    Tim Heidmann
    Created     March  1, 1993
    Last Edit   Oct   22, 1993
*/

#ifndef TRUE
#define TRUE 1L
#endif
#ifndef FALSE
#define FALSE 0L
#endif

#include <sys/time.h>

/* VLAN error codes, in vlan_errno */
#define VLAN_CANT_OPEN        1
#define VLAN_TCGETA           2
#define VLAN_TCSETA           3
#define VLAN_PARSE_TIMECODE   4
#define VLAN_BAD_TIMEOUT      5
#define VLAN_READ_COINCIDENCE 6
#define VLAN_TIMEOUT          7
#define VLAN_CAPTURE_DEAD     8

typedef struct {
    union {
	struct {
	    unsigned char esc, type, nBytes;	/* 1-3             */
	} generic;

	struct {
	    unsigned char esc, type, nBytes;	/* 1-3             */
	    unsigned mType : 3, node : 5;	/* 4    body 1     */
	    unsigned char f0, f1, f2;		/* 5-7  body 2-4   */
	    unsigned char se;			/* 8    body 5     */
	    unsigned code : 2,
		     byte7type : 2,
		     locked : 1,
		     varSpeed : 1,
		     dropFrame : 1,
		     tcValid : 1;		/* 9    body 6     */
	    unsigned char byte7;		/* 10   body 7     */
	    unsigned : 4,
		     idChanged : 1,
		     mediaChanged : 1,
		     searchCompleted : 1,
		     editCompleted : 1;		/* 11   body 8     */
	    unsigned : 8;			/* 12   pad to 12  */
	} status;
    } u;
} vlan_esc_struct;


extern int vlan_sending, vlan_receiving, vlan_debug, vlan_errno;
extern char vlan_port[];

extern int vlan_InitPort();
extern char * vlan_SendCmd(char c[]);
extern int vlan_Coincidence(int ds, char *ch);

extern char *vlan_FtoTC(int iFrame);
extern int vlan_TCtoF(char *tc);
extern char *vlan_neatTC(char *tc);

extern void vlan_perror(char *str);
void vlan_Send(char c[]);
int vlan_Recv(char **c, vlan_esc_struct **e, struct timeval *ts);
